#include <opencv2/core/core.hpp>
#include <opencv2/highgui/highgui.hpp>
#include <opencv2/imgproc/imgproc.hpp>
#include <iostream>
#include <stdio.h>
#include <vector>
#include <algorithm>
#include <chrono>


//yolov3---
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <vip_lite.h>

#define _BASETSD_H
#include "opencv2/opencv.hpp"
extern "C" {
#include "vnn_global.h"
#include "vnn_pre_process.h"
//#include "vnn_post_process.h"
}
//yolov3---


using namespace std;

enum Yolov5OutType
{
    p8_type     = 1,
    p16_type    = 2,
    p32_type    = 3,
};

struct Object
{
    cv::Rect_<float> rect;
    int label;
    float prob;
};

typedef struct blob
{
    int w;
    float *data;
} blob;


int get_tensor_data(string file_path, float *data)
{
    int len = 0;
    static float *memory = NULL;
    static int max_len = 1*1024*1024;

    if (memory == NULL)
        memory = (float *)malloc(max_len * sizeof(float));

    FILE *fp = NULL;
    if ((fp = fopen(file_path.c_str(), "r")) == NULL)
    {
        cout << "open tensor file error ! file name : " << file_path << endl;
        exit(-1);
    }

    int file_len = 0;
    while (!feof(fp))
    {
        fscanf(fp, "%f ", &memory[len++]);
    }

    memcpy(data, memory, len * sizeof(float));
    fclose(fp);

    if (len == 0 || data == NULL)
    {
        cout << "read tensor error happened ! " << "len : " << len << " data address: " << *data << endl;
        exit(-1);
    }

    return len;
}

static inline float sigmoid(float x)
{
    return static_cast<float>(1.f / (1.f + exp(-x)));
}

static inline float intersection_area(const Object& a, const Object& b)
{
    cv::Rect_<float> inter = a.rect & b.rect;
    return inter.area();
}

static void qsort_descent_inplace(std::vector<Object>& faceobjects, int left, int right)
{
    int i = left;
    int j = right;
    float p = faceobjects[(left + right) / 2].prob;

    while (i <= j)
    {
        while (faceobjects[i].prob > p)
            i++;

        while (faceobjects[j].prob < p)
            j--;

        if (i <= j)
        {
            // swap
            std::swap(faceobjects[i], faceobjects[j]);

            i++;
            j--;
        }
    }

#pragma omp parallel sections
    {
#pragma omp section
        {
            if (left < j) qsort_descent_inplace(faceobjects, left, j);
        }
#pragma omp section
        {
            if (i < right) qsort_descent_inplace(faceobjects, i, right);
        }
    }
}

static void qsort_descent_inplace(std::vector<Object>& faceobjects)
{
    if (faceobjects.empty())
        return;

    qsort_descent_inplace(faceobjects, 0, faceobjects.size() - 1);
}

static void nms_sorted_bboxes(const std::vector<Object>& faceobjects, std::vector<int>& picked, float nms_threshold)
{
    picked.clear();

    const int n = faceobjects.size();

    std::vector<float> areas(n);
    for (int i = 0; i < n; i++)
    {
        areas[i] = faceobjects[i].rect.area();
    }

    for (int i = 0; i < n; i++)
    {
        const Object& a = faceobjects[i];

        int keep = 1;
        for (int j = 0; j < (int)picked.size(); j++)
        {
            const Object& b = faceobjects[picked[j]];

            // intersection over union//交并集
            float inter_area = intersection_area(a, b);
            float union_area = areas[i] + areas[picked[j]] - inter_area;
            // float IoU = inter_area / union_area
            if (inter_area / union_area > nms_threshold)
                keep = 0;
        }

        if (keep)
            picked.push_back(i);
    }
}

static void generate_proposals(int stride, const float* feat, float prob_threshold, std::vector<Object>& objects,
                               int letterbox_cols, int letterbox_rows)
{
    static float anchors[18] = {10, 13, 16, 30, 33, 23, 30, 61, 62, 45, 59, 119, 116, 90, 156, 198, 373, 326};

    int anchor_num = 3;
    int feat_w = letterbox_cols / stride;
    int feat_h = letterbox_rows / stride;
    int cls_num = 80;
    int anchor_group;
    if (stride == 8)
        anchor_group = 1;
    if (stride == 16)
        anchor_group = 2;
    if (stride == 32)
        anchor_group = 3;
    for (int h = 0; h <= feat_h - 1; h++)
    {
        for (int w = 0; w <= feat_w - 1; w++)
        {
            for (int a = 0; a <= anchor_num - 1; a++)
            {
                //process cls score
                int class_index = 0;
                float class_score = -FLT_MAX;
                for (int s = 0; s <= cls_num - 1; s++)
                {
                    float score = feat[a * feat_w * feat_h * (cls_num + 5) + h * feat_w * (cls_num + 5) + w * (cls_num + 5) + s + 5];
                    if (score > class_score)
                    {
                        class_index = s;
                        class_score = score;
                    }
                }
                //process box score
                float box_score = feat[a * feat_w * feat_h * (cls_num + 5) + (h * feat_w) * (cls_num + 5) + w * (cls_num + 5) + 4];
                float final_score = sigmoid(box_score) * sigmoid(class_score);
                if (final_score >= prob_threshold)
                {
                    int loc_idx = a * feat_h * feat_w * (cls_num + 5) + h * feat_w * (cls_num + 5) + w * (cls_num + 5);
                    float dx = sigmoid(feat[loc_idx + 0]);
                    float dy = sigmoid(feat[loc_idx + 1]);
                    float dw = sigmoid(feat[loc_idx + 2]);
                    float dh = sigmoid(feat[loc_idx + 3]);
                    float pred_cx = (dx * 2.0f - 0.5f + w) * stride;
                    float pred_cy = (dy * 2.0f - 0.5f + h) * stride;
                    float anchor_w = anchors[(anchor_group - 1) * 6 + a * 2 + 0];
                    float anchor_h = anchors[(anchor_group - 1) * 6 + a * 2 + 1];
                    float pred_w = dw * dw * 4.0f * anchor_w;
                    float pred_h = dh * dh * 4.0f * anchor_h;
                    float x0 = pred_cx - pred_w * 0.5f;
                    float y0 = pred_cy - pred_h * 0.5f;
                    float x1 = pred_cx + pred_w * 0.5f;
                    float y1 = pred_cy + pred_h * 0.5f;

                    Object obj;
                    obj.rect.x = x0;
                    obj.rect.y = y0;
                    obj.rect.width = x1 - x0;
                    obj.rect.height = y1 - y0;
                    obj.label = class_index;
                    obj.prob = final_score;
                    objects.push_back(obj);
                }
            }
        }
    }
}




//----------------------------------------------------------------------------------------------//

/*-------------------------------------------
        Macros and Variables
-------------------------------------------*/
const char *usage =
    "Usage:\nnbg_name input_data1 input_data2...";
/*-------------------------------------------
                  Functions
-------------------------------------------*/
#define BILLION 1000000000
static vip_uint64_t get_perf_count()
{
    struct timespec ts;

    clock_gettime(CLOCK_MONOTONIC, &ts);

    return (vip_uint64_t)((vip_uint64_t)ts.tv_nsec + (vip_uint64_t)ts.tv_sec * BILLION);
}
/* Initialize network items */  //初始化网络项目
vip_status_e vnn_InitNetworkItem(vip_network_items **network_items, int argc, char **argv)
{
    /*
     * argv0:   execute file
     * argv1:   nbg file
     * argv2~n: inputs n files
     */
    vip_status_e status = VIP_SUCCESS;
    vip_network_items *nnItems = NULL;
    const char *file_name = NULL;
    int input_num = 0, i = 0;
    char **inputs = NULL;
    int name_len = 0;

    file_name = (const char *)argv[1];//得到文件名
    input_num = argc - 2;
    if (input_num <= 0)
    {
        status = VIP_ERROR_INVALID_ARGUMENTS;
        goto final;
    }
    inputs = argv + 2;

    nnItems = (vip_network_items *)malloc(sizeof(vip_network_items));
    memset(nnItems, 0, sizeof(vip_network_items));

    name_len = strlen(file_name);
    if (name_len <= 0)
    {
        if (nnItems)
        {
            free(nnItems);
            nnItems = NULL;
        }
        status = VIP_ERROR_INVALID_ARGUMENTS;
        goto final;
    }
    nnItems->nbg_name = (char *)malloc(name_len + 1);
    memset(nnItems->nbg_name, 0, name_len + 1);
    strcpy(nnItems->nbg_name, file_name);
    nnItems->input_count = input_num;
    nnItems->input_names = (char **)malloc(sizeof(char *) * input_num);
    for (i = 0; i < input_num; i++)
    {
        nnItems->input_names[i] = inputs[i];
    }

    *network_items = nnItems;

final:
    return status;
}
/* Create the neural network */  //初始化网络项创建神经网络
static vip_status_e vnn_CreateNeuralNetwork(vip_network_items *network_items)
{
    vip_status_e status = VIP_SUCCESS;
    vip_uint64_t tmsStart, tmsEnd;
    float msVal, usVal;

    tmsStart = get_perf_count();
    status = vip_create_network(network_items->nbg_name, 0,
                                VIP_CREATE_NETWORK_FROM_FILE, &network_items->network);
    _CHECK_STATUS(status, final);
    tmsEnd = get_perf_count();
    msVal = (float)(tmsEnd - tmsStart) / 1000000;
    usVal = (float)(tmsEnd - tmsStart) / 1000;
    printf("Create Neural Network: %.2fms or %.2fus\n", msVal, usVal);

final:
    return status;
}
/* Pre process the input/output data */  //预处理输入/输出数据
static vip_status_e vnn_PreProcessNeuralNetwork(vip_network_items *network_items)
{
    vip_status_e status = VIP_SUCCESS;

    /* Create input/output buffers, prepare network */
    //创建输入/输出缓冲区，准备网络
    status = vnn_CreateInOutBufPrepareNetwork(network_items);
    _CHECK_STATUS(status, final);
    /* Set input/output buffers */
    status = vnn_SetNetworkInOut(network_items);
    _CHECK_STATUS(status, final);

final:
    return status;
}
/* Run the neural network */  //运行神经网络
vip_status_e vnn_RunNeuralNetwork(vip_network_items *network_items)
{
    vip_status_e status = VIP_SUCCESS;
    vip_int32_t i = 0, loop = 1;
    char *loop_s;
    vip_uint64_t tmsStart, tmsEnd, sigStart, sigEnd;
    float msVal, usVal;

    loop_s = getenv("VNN_LOOP_TIME");
    if (loop_s)
    {
        loop = atoi(loop_s);
    }

    /* Run network */
    tmsStart = get_perf_count();
    printf("Start run graph [%d] times...\n", loop);
    for (i = 0; i < loop; i++)
    {
        sigStart = get_perf_count();
        status = vip_run_network(network_items->network);
        _CHECK_STATUS(status, final);
        sigEnd = get_perf_count();
        msVal = (float)(sigEnd - sigStart) / 1000000;
        usVal = (float)(sigEnd - sigStart) / 1000;
        printf("Run the %d time: %.2fms or %.2fus\n", (i + 1), msVal, usVal);
    }
    tmsEnd = get_perf_count();
    msVal = (float)(tmsEnd - tmsStart) / 1000000;
    usVal = (float)(tmsEnd - tmsStart) / 1000;
    printf("vip run network execution time:\n");
    printf("Total   %.2fms or %.2fus\n", msVal, usVal);
    printf("Average %.2fms or %.2fus\n", (float)msVal / loop, (float)usVal / loop);



final:
    return status;
}
static vip_float_t uint8_to_fp32(vip_uint8_t val, vip_int32_t zeroPoint, vip_float_t scale)
{
    vip_float_t result = 0.0f;

    result = (val - (vip_uint8_t)zeroPoint) * scale;
    return result;
}


vip_status_e destroy_network(vip_network_items *network_items)
{
    vip_status_e status = VIP_SUCCESS;
    int i = 0;

    status = vip_finish_network(network_items->network);
    _CHECK_STATUS(status, final);
    status = vip_destroy_network(network_items->network);
    _CHECK_STATUS(status, final);

    for (i = 0; i < network_items->input_count; i++)
    {
        status = vip_destroy_buffer(network_items->input_buffers[i]);
        _CHECK_STATUS(status, final);
    }
    if (network_items->input_buffers)
    {
        free(network_items->input_buffers);
        network_items->input_buffers = VIP_NULL;
    }
    for (i = 0; i < network_items->output_count; i++)
    {
        status = vip_destroy_buffer(network_items->output_buffers[i]);
        _CHECK_STATUS(status, final);
    }
    if (network_items->output_buffers)
    {
        free(network_items->output_buffers);
        network_items->output_buffers = VIP_NULL;
    }

final:
    return status;
}

void destroy_network_items(
    vip_network_items *network_items)
{
    if (network_items->nbg_name)
    {
        free(network_items->nbg_name);
        network_items->nbg_name = VIP_NULL;
    }
    if (network_items->input_names)
    {
        free(network_items->input_names);
        network_items->input_names = VIP_NULL;
    }
    if (network_items)
    {
        free(network_items);
    }
}

std::vector<float> p8_data;//定义 浮点型 变量（p8_data ）大小（1*3*52*52*85）个
std::vector<float> p16_data;
std::vector<float> p32_data;

vip_status_e save_output_data(vip_network_items *network_items)
{
    vip_status_e status = VIP_SUCCESS;
    int i = 0;
#define _DUMP_FILE_LENGTH 1028
#define _DUMP_SHAPE_LENGTH 128
    char filename[_DUMP_FILE_LENGTH] = {0}, shape[_DUMP_SHAPE_LENGTH] = {0};
    int buff_size = 0;
    void *out_data = NULL;
    vip_buffer_create_params_t buf_param;
    //printf("vnn_post_process.c___98____\n");
    blob blobs[3];
    printf("###network_items->output_count###%d###\n",network_items->output_count);//3
    for (i = 0; i < network_items->output_count; i++)
    {
        //printf("vnn_post_process.c___102____%d\n",i);
        buff_size = vip_get_buffer_size(network_items->output_buffers[i]);
        if (buff_size <= 0)
        {
            status = VIP_ERROR_IO;
            return status;
        }
        //printf("vnn_post_process.c___109____%d\n",i);
        memset(&buf_param, 0, sizeof(buf_param));
        status = vip_query_output(network_items->network, i,
                                  VIP_BUFFER_PROP_DATA_FORMAT, &buf_param.data_format);
        _CHECK_STATUS(status, final);
        //printf("vnn_post_process.c___114____%d\n",i);
        status = vip_query_output(network_items->network, i,
                                  VIP_BUFFER_PROP_NUM_OF_DIMENSION, &buf_param.num_of_dims);
        _CHECK_STATUS(status, final);
        status = vip_query_output(network_items->network, i,
                                  VIP_BUFFER_PROP_SIZES_OF_DIMENSION, buf_param.sizes);
        _CHECK_STATUS(status, final);
        status = vip_query_output(network_items->network, i,
                                  VIP_BUFFER_PROP_TF_ZERO_POINT, &buf_param.quant_data.affine.zeroPoint);
        _CHECK_STATUS(status, final);
        status = vip_query_output(network_items->network, i,
                                  VIP_BUFFER_PROP_TF_SCALE, &buf_param.quant_data.affine.scale);
        _CHECK_STATUS(status, final);
        out_data = vip_map_buffer(network_items->output_buffers[i]);

        vip_int32_t zeroPoint = buf_param.quant_data.affine.zeroPoint;
        vip_float_t scale = buf_param.quant_data.affine.scale;
        vip_uint8_t *data = (vip_uint8_t *)out_data;
        float *fp_data = (float *)malloc(buff_size * sizeof(float));
        printf("data_format=%d buff_size=%d\n", buf_param.data_format, buff_size); // VIP_BUFFER_FORMAT_UINT8=2
        for (int j = 0; j < buff_size; j++)
        {
            fp_data[j] = uint8_to_fp32(*data, zeroPoint, scale);
            if (i==0)
            {
                /*
//                vector<int> v1;  //空vector对象
//                for (int i = 0; i != 100; ++i)
//                    v1.push_back(i);   //依次把整数值放到v1的尾端
//                for (int i = 0; i != 100; ++i)
//                    //v1.push_back(i);   //依次把整数值放到v1的尾端
//                    printf("v1.push_back(i)----#%d----111-----\n",v1[i]);//a.insert(a.end(),10,5) 在末尾插入10个值为5的元素
//                //循环结束后v1有100个元素，值从0到99
                */
                p8_data.push_back(fp_data[j]);
            }
            if (i==1)
            {
                p16_data.push_back(fp_data[j]);                      //a.insert(a.end(),10,5) 在末尾插入10个值为5的元素
            }
            if (i==2)
            {
                 p32_data.push_back(fp_data[j]);                      //a.insert(a.end(),10,5) 在末尾插入10个值为5的元素
            }
            data += 1; // VIP_BUFFER_FORMAT_UINT8 one byte
        }
//        for (int i = 0; i<p32_data.size(); i++)
//            printf("--111-----#%f----111-----\n",p32_data.data()[i]);

        /*
        for (int j = 0; j < buff_size; j++)
        {
            //printf("-----%f------\n",fp_data[j] );
            char chFloat[20];
            cp_data[j]=sprintf(chFloat,"%f",fp_data[j]);
            //cp_data[j] = uint8_to_fp32(*data, zeroPoint, scale);
            //printf("-----------%s--------------------\n",cp_data[j] );
            data += 1; // VIP_BUFFER_FORMAT_UINT8 one byte
        }
        */
        blob temp_blob = {0, NULL};
        temp_blob.data = fp_data;
        blobs[i] = temp_blob;
        //save_file(filename, cp_data, buff_size);
    }

final:
    return status;
}
static int detect_yolov5(const cv::Mat& bgr, std::vector<Object>& objects)
{
    std::chrono::steady_clock::time_point Tbegin, Tend;
    Tbegin = std::chrono::steady_clock::now();
//    std::vector<float> p8_data(1*3*52*52*85);//定义 浮点型 变量（p8_data ）大小（1*3*52*52*85）个
//    std::vector<float> p16_data(1*3*26*26*85);
//    std::vector<float> p32_data(1*3*13*13*85);

    // set default letterbox size
    int letterbox_rows = 416;
    int letterbox_cols = 416;
     /* postprocess */
    const float prob_threshold = 0.5f;
    const float nms_threshold = 0.45f;
    std::vector<Object> proposals;
    std::vector<Object> objects8;
    std::vector<Object> objects16;
    std::vector<Object> objects32;
//    for (int i = 0; i<8; i++)
//        printf("-------#%f---------\n",p8_data.data()[i]);
    generate_proposals(32, p32_data.data(), prob_threshold, objects32, letterbox_cols, letterbox_rows);
    proposals.insert(proposals.end(), objects32.begin(), objects32.end());
    generate_proposals(16, p16_data.data(), prob_threshold, objects16, letterbox_cols, letterbox_rows);
    proposals.insert(proposals.end(), objects16.begin(), objects16.end());
    generate_proposals(8, p8_data.data(), prob_threshold, objects8, letterbox_cols, letterbox_rows);
    proposals.insert(proposals.end(), objects8.begin(), objects8.end());

    qsort_descent_inplace(proposals);
    std::vector<int> picked;
    nms_sorted_bboxes(proposals, picked, nms_threshold);

    /* yolov5 draw the result *///绘制结果
    float scale_letterbox;
    int resize_rows;
    int resize_cols;
    if ((letterbox_rows * 1.0 / bgr.rows) < (letterbox_cols * 1.0 / bgr.cols))
    {
        scale_letterbox = letterbox_rows * 1.0 / bgr.rows;
    }
    else
    {
        scale_letterbox = letterbox_cols * 1.0 / bgr.cols;
    }
    resize_cols = int(scale_letterbox * bgr.cols);
    resize_rows = int(scale_letterbox * bgr.rows);

    int tmp_h = (letterbox_rows - resize_rows) / 2;
    int tmp_w = (letterbox_cols - resize_cols) / 2;

    float ratio_x = (float)bgr.rows / resize_rows;
    float ratio_y = (float)bgr.cols / resize_cols;

    int count = picked.size();
    fprintf(stderr, "detection num: %d\n", count);

    objects.resize(count);
    for (int i = 0; i < count; i++)
    {
        objects[i] = proposals[picked[i]];
        float x0 = (objects[i].rect.x);
        float y0 = (objects[i].rect.y);
        float x1 = (objects[i].rect.x + objects[i].rect.width);
        float y1 = (objects[i].rect.y + objects[i].rect.height);

        x0 = (x0 - tmp_w) * ratio_x;
        y0 = (y0 - tmp_h) * ratio_y;
        x1 = (x1 - tmp_w) * ratio_x;
        y1 = (y1 - tmp_h) * ratio_y;

        x0 = std::max(std::min(x0, (float)(bgr.cols - 1)), 0.f);
        y0 = std::max(std::min(y0, (float)(bgr.rows - 1)), 0.f);
        x1 = std::max(std::min(x1, (float)(bgr.cols - 1)), 0.f);
        y1 = std::max(std::min(y1, (float)(bgr.rows - 1)), 0.f);

        objects[i].rect.x = x0;
        objects[i].rect.y = y0;
        objects[i].rect.width = x1 - x0;
        objects[i].rect.height = y1 - y0;
    }

    Tend = std::chrono::steady_clock::now();
    float f = std::chrono::duration_cast <std::chrono::milliseconds> (Tend - Tbegin).count();

    std::cout << "time : " << f/1000.0 << " Sec" << std::endl;

    return 0;
}

static void draw_objects(const cv::Mat& bgr, const std::vector<Object>& objects)
{
    static const char* class_names[] = {
        "person", "bicycle", "car", "motorcycle", "airplane", "bus", "train", "truck", "boat", "traffic light",
        "fire hydrant", "stop sign", "parking meter", "bench", "bird", "cat", "dog", "horse", "sheep", "cow",
        "elephant", "bear", "zebra", "giraffe", "backpack", "umbrella", "handbag", "tie", "suitcase", "frisbee",
        "skis", "snowboard", "sports ball", "kite", "baseball bat", "baseball glove", "skateboard", "surfboard",
        "tennis racket", "bottle", "wine glass", "cup", "fork", "knife", "spoon", "bowl", "banana", "apple",
        "sandwich", "orange", "broccoli", "carrot", "hot dog", "pizza", "donut", "cake", "chair", "couch",
        "potted plant", "bed", "dining table", "toilet", "tv", "laptop", "mouse", "remote", "keyboard", "cell phone",
        "microwave", "oven", "toaster", "sink", "refrigerator", "book", "clock", "vase", "scissors", "teddy bear",
        "hair drier", "toothbrush"};

    cv::Mat image = bgr.clone();

    for (size_t i = 0; i < objects.size(); i++)
    {
        const Object& obj = objects[i];

        fprintf(stderr, "%2d: %3.0f%%, [%4.0f, %4.0f, %4.0f, %4.0f], %s\n", obj.label, obj.prob * 100, obj.rect.x,
                obj.rect.y, obj.rect.x + obj.rect.width, obj.rect.y + obj.rect.height, class_names[obj.label]);

        cv::rectangle(image, obj.rect, cv::Scalar(255, 0, 0));

        char text[256];
        sprintf(text, "%s %.1f%%", class_names[obj.label], obj.prob * 100);

        int baseLine = 0;
        cv::Size label_size = cv::getTextSize(text, cv::FONT_HERSHEY_SIMPLEX, 0.5, 1, &baseLine);

        int x = obj.rect.x;
        int y = obj.rect.y - label_size.height - baseLine;
        if (y < 0)
            y = 0;
        if (x + label_size.width > image.cols)
            x = image.cols - label_size.width;

        cv::rectangle(image, cv::Rect(cv::Point(x, y), cv::Size(label_size.width, label_size.height + baseLine)),
                      cv::Scalar(255, 255, 255), -1);

        cv::putText(image, text, cv::Point(x, y + label_size.height), cv::FONT_HERSHEY_SIMPLEX, 0.5,
                    cv::Scalar(0, 0, 0));
    }

    cv::imwrite("yolov5_out.jpg", image);
}

/* Post process output data */  //后处理输出数据
vip_status_e vnn_PostProcessNeuralNetwork(vip_network_items *network_items)
{
    return save_output_data(network_items);
}

vip_status_e vnn_ReleaseNeuralNetwork(vip_network_items *network_items)
{
    vip_status_e status = VIP_SUCCESS;

    status = destroy_network(network_items);
    _CHECK_STATUS(status, final);
    destroy_network_items(network_items);
    status = vip_destroy();
    _CHECK_STATUS(status, final);

final:
    return status;
}

int main(int argc, char** argv)
{
    // 设置SFTP服务器地址、用户名和密码
    string sftp_server = "192.168.134.148";
    string sftp_user = "fawen";
    string sftp_pass = "8888";

    // 设置本地文件路径和远程文件路径
    string local_file = "/root/yolov5_out.jpg";
    string remote_file = "yolov5_out.jpg";
    int r;
    string sftp_command = "echo 'put " + local_file + " " + remote_file + "' | sshpass -p '8888' sftp -oPort=22 -o StrictHostKeyChecking=yes " + sftp_user + "@" + sftp_server;
    vip_status_e status = VIP_SUCCESS;
    vip_network_items *network_items = VIP_NULL;
    printf("argc %d\n", argc);
    printf("argv[0] %s\n", argv[0]);
    printf("argv[1] %s\n", argv[1]);
    printf("argv[2] %s\n", argv[2]);
    const char* imagepath = argv[2];
    cv::Mat m = cv::imread(imagepath, 1);
    if (m.empty())
    {
        fprintf(stderr, "cv::imread %s failed\n", imagepath);
        return -1;
    }
    std::vector<Object> objects;


    //return 0;
    if (argc < 3)
    {
        fprintf(stderr, "Usage: %s [imagepath]\n", argv[0]);
        return -1;
    }
    printf("输入参数个数 %d!\n", argc);
    printf("输入参数argv %s--%s--%s\n", argv[0],argv[1],argv[2]);
    if (argc < 3)
    {
        printf("%s\n", usage);
        printf("Arguments count %d is incorrect!\n", argc);
        return -1;
    }

    /* Initialize  vip lite */
    status = vip_init(13 * 1024 * 1024);
    _CHECK_STATUS(status, final);
    printf("Initialize  vip lite---ok---!\n");
    /* Initialize network items */  //初始化网络项目
    status = vnn_InitNetworkItem(&network_items, argc, argv);
    _CHECK_STATUS(status, final);
    printf("Initialize network items---ok---!\n");
    /* Create the neural network */  //初始化网络项创建神经网络
    status = vnn_CreateNeuralNetwork(network_items);
    _CHECK_STATUS(status, final);
    printf("Create the neural network ---ok---!\n");
    /* Pre process the input/output data */  //预处理输入/输出数据
    status = vnn_PreProcessNeuralNetwork(network_items);
    _CHECK_STATUS(status, final);
    printf("Pre process the input/output data ---ok---!\n");
    /* Run the neural network */  //运行神经网络
     status = vnn_RunNeuralNetwork(network_items);
     _CHECK_STATUS(status, final);
    printf("Run the neural network ---ok---!\n");
    /* Post process output data */  //后处理输出数据
    status = vnn_PostProcessNeuralNetwork(network_items);
    _CHECK_STATUS(status, final);

//----------------------------------------------------------------------------------------------//
    detect_yolov5(m, objects);

    draw_objects(m, objects);
    //system("ls -al /etc/passwd /etc/shadow");
    //system("reboot");
    //r=system("sshpass -p '8888' scp  -P 38940 /root/yolov5_out.jpg  fawen@73l984h497.zicp.fun:/yolov5_out.jpg");
    //-oPort=22 -o StrictHostKeyChecking=yes
    //echo put   /root/yolov5_out.jpg  yolov5_out.jpg  | sshpass -p '8888' sftp -oPort=22 -o StrictHostKeyChecking=yes fawen@192.168.134.148

    // 执行SFTP命令
     r = system(sftp_command.c_str());
    //r=system("sshpass -p '8888' scp -P 22 -o StrictHostKeyChecking=yes /root/yolov5_out.jpg fawen@192.168.134.148:/yolov5_out.jpg");//
    printf("传输状态%d--\n",r);

final:

    /* Destroy resources *///终止 VIPLite 驱动程序，释放 vip_init（）请求的资源，并关闭 vip hard
    status = vnn_ReleaseNeuralNetwork(network_items);
    printf("Destroy resources ---ok---!\n");
    return status;


}
